﻿B4J=true
Group=Default Group\DataBase
ModulesStructureVersion=1
Type=Class
Version=10
@EndOfDesignText@
'This modules provides the interface to the database(s) 
Sub Class_Globals
	Private fx As JFX
	Private rtdb As SQL
End Sub

'======================================================================================================
'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize(dbPath As String, dbFileName As String) As Boolean
	
	Try
		rtdb.InitializeSQLite(dbPath, dbFileName, False)
	Catch
		Log(dbPath & dbFileName & "  " & LastException)
		Return True
	End Try
	
	If rtdb.IsInitialized Then
		setDbToWAL(rtdb)
		checkPointDataBase(rtdb)
	End If
	Return False
End Sub

'======================================================================================================
private Sub setDbToWAL(db As SQL) As Boolean
'	errRec.fcnName = "setDbToWAL"
	Dim qryStr As String = $"PRAGMA journal_mode = WAL"$
	
	Try
		db.ExecNonQuery(qryStr)
	Catch
		Log("clkDatabase Line 32" & LastException)
		Return False
	End Try
	Return True
End Sub


'======================================================================================================
Public Sub checkPointDataBase(db As SQL) As Boolean
	Dim qryStr As String = $"PRAGMA wal_checkpoint(TRUNCATE)"$
	
	Try
		db.ExecNonQuery(qryStr)
	Catch
		Return False
	End Try
	Return True
End Sub


'============================================================================================
Public Sub getSignalSourceRecords(deleted As Int) As ResultSet
	Dim dbRes As ResultSet
	
	Dim pars As List
	pars.Initialize
	pars.Add(deleted)

	Dim qryStr As String
		
	qryStr = _
	$"SELECT 
       SignalSource_ID, 		
       SignalName, 
       SignalBandStart_MHz, 
       SignalBandStop_MHz, 
       Comment, 
       DateCreated, 
       DateUsed, 
       InUse, 
       DateAccessed,
	   NoPoints,
	   SampleTime,
	   LNA_On,
	   LNA_Gain,
	   Attenuator,
	   RBW_Khz,
	   RefLevel,
	   NoSpectrumAvgs,
	   RoutineDataFrequencyN
	FROM   tblSignalSource
	WHERE Deleted = ?
	ORDER  BY
	      InUse DESC,
	      DateAccessed DESC,
	      DateUsed DESC,
	      DateCreated DESC;"$
	Try
		dbRes = rtdb.ExecQuery2(qryStr, pars)
	Catch
		Log(LastException)
	End Try
	Return dbRes
End Sub


'============================================================================================
Public Sub getLatestSignalSourceRecord As Int
	Dim dbRes As ResultSet

	Dim qryStr As String
		
	qryStr = _
	$"SELECT 
       SignalSource_ID
	FROM   tblSignalSource
	ORDER  BY
	      DateCreated DESC
		  Limit 1;"$
	Try
		dbRes = rtdb.ExecQuery(qryStr)
	Catch
		Log(LastException)
	End Try
	
	Return dbRes.GetInt2(0)

End Sub


'============================================================================================
Public Sub clearSignalSrcRecInUse As Boolean
	Dim qryStr As String
		
	qryStr = _
	$"UPDATE tblSignalSource
      SET InUse = 0
	WHERE  InUse == 1"$
	Try
		rtdb.ExecNonQuery(qryStr)
	Catch
		Log(LastException)
		Return True
	End Try
	Return False
End Sub


'============================================================================================
Public Sub setSignalSrcRecInUse(SigSrcID As Int) As Boolean
	Dim qryStr As String
	
	Dim argList As List
	argList.Initialize2(Array As Int (SigSrcID))
		
	qryStr = _
	$"UPDATE tblSignalSource
      SET InUse = 1
	WHERE  SignalSource_ID == ?"$
	Try
		rtdb.ExecNonQuery2(qryStr, argList)
	Catch
		Log(LastException)
		Return True
	End Try
	Return False
End Sub


'============================================================================================
Public Sub setSignalSrcRecNewestInUse As Boolean
	Dim qryStr As String
	
	qryStr = _
	$"UPDATE tblSignalSource
     	SET InUse = 1
	 where SignalSource_ID =
		(select SignalSource_ID
		from tblSignalSource
		where Deleted = 0
		ORDER BY DateAccessed DESC
		LIMIT 1)"$
	Try
		rtdb.ExecNonQuery(qryStr)
	Catch
		Log(LastException)
		Return True
	End Try
	Return False
End Sub

'============================================================================================
Public Sub updateSignalSourceRecord As Boolean
	Dim pars As List
	pars.Initialize
	
	pars.Add(Main.Band_Data.SigSrcCurrentRecord.SignalName)
	pars.Add(Main.Band_Data.SigSrcCurrentRecord.SignalBandStart_MHz)
	pars.Add(Main.Band_Data.SigSrcCurrentRecord.SignalBandStop_MHz)
	pars.Add(Main.Band_Data.SigSrcCurrentRecord.Comment)
	pars.Add(1)
	pars.Add(Main.Band_Data.SigSrcCurrentRecord.DateAccessed)
	pars.Add(Main.Band_Data.SigSrcCurrentRecord.NoPoints)
	pars.Add(Main.Band_Data.SigSrcCurrentRecord.SampleTime)
	pars.Add(Main.Band_Data.SigSrcCurrentRecord.LNA_On)
	pars.Add("auto")
	pars.Add(Main.Band_Data.SigSrcCurrentRecord.Attenuator)
	pars.Add(Main.Band_Data.SigSrcCurrentRecord.RBW_Khz)
	pars.Add(Main.Band_Data.SigSrcCurrentRecord.RefLevel)
	pars.Add(Main.Band_Data.SigSrcCurrentRecord.NoSpectrumAvgs)
	pars.Add(Main.Band_Data.SigSrcCurrentRecord.RoutineDataFrequencyN)
	pars.Add(Main.Band_Data.SigSrcCurrentRecord.SignalSource_ID)
	Dim qryStr As String
		
	qryStr = _
	$"UPDATE tblSignalSource 
	SET SignalName = ?, 
		SignalBandStart_MHz = ?,
       	SignalBandStop_MHz = ?,
       	Comment = ?, 
       	InUse  = ?, 
       	DateAccessed = ?,
       	NoPoints  = ?,
       	SampleTime = ?,
       	LNA_On = ?,
		LNA_Gain = ?,
       	Attenuator  = ?,
       	RBW_Khz  = ?,
       	RefLevel  = ?,
       	NoSpectrumAvgs = ?,
       	RoutineDataFrequencyN  = ?

	where  SignalSource_ID = ?	"$

	Try
		rtdb.ExecNonQuery2(qryStr, pars)
	Catch
		Log(LastException)
		Return True
	End Try
	Return False
End Sub


'============================================================================================
Public Sub doesSignalSrcRecHaveExceptions(sigSrcID As Int) As Int
	Dim dbRes As ResultSet
	
	Dim pars As List
	pars.Initialize
	pars.Add(sigSrcID)

	Dim qryStr As String
		
	qryStr = _
	$"SELECT 
       count([tblChannelExceptionData].[MeanSignal_dBm])
	FROM   [tblSignalSource]
       INNER JOIN [tblChannelExceptionData] ON [tblChannelExceptionData].[SignalSource_ID] = [tblSignalSource].[SignalSource_ID]
	WHERE  [tblSignalSource].[SignalSource_ID] = ?;"$
	Try
		dbRes = rtdb.ExecQuery2(qryStr, pars)
	Catch
		Log(LastException)
	End Try
	
	Return dbRes.GetInt2(0)
End Sub


'============================================================================================
Public Sub doesSignalSrcRecHaveBands(sigSrcID As Int) As Int
	Dim dbRes As ResultSet
	
	Dim pars As List
	pars.Initialize
	pars.Add(sigSrcID)

	Dim qryStr As String
		
	qryStr = _
	$"SELECT count([main].[tblMonitoredChannel].[MonitoredCh_ID])
	FROM   [main].[tblSignalSource]
       INNER JOIN [main].[tblMonitoredChannel] ON [main].[tblSignalSource].[SignalSource_ID] = [main].[tblMonitoredChannel].[SignalSource_ID]
	WHERE  [main].[tblSignalSource].[SignalSource_ID] = ?;"$
	Try
		dbRes = rtdb.ExecQuery2(qryStr, pars)
	Catch
		Log(LastException)
	End Try
	
	Return dbRes.GetInt2(0)
End Sub


'============================================================================================
Public Sub deleteSignalSourceRecord(SignalSource_ID As Int, delete As Int) As Boolean
	Dim pars As List
	pars.Initialize
	
	pars.Add(delete)
	pars.Add(SignalSource_ID)

	Dim qryStr As String
		
	qryStr = _
	$"UPDATE tblSignalSource 
	SET Deleted = ?

	where  SignalSource_ID = ?"$

	Try
		rtdb.ExecNonQuery2(qryStr, pars)
	Catch
		Log(LastException)
		Return True
	End Try
	Return False
End Sub



'============================================================================================
Public Sub deleteRunIndexRecord(SignalSource_ID As Int, delete As Int) As Boolean
	Dim pars As List
	pars.Initialize
	
	pars.Add(delete)
	pars.Add(SignalSource_ID)

	Dim qryStr As String
		
	qryStr = _
	$"UPDATE tblBandExceptionRunIndex 
	SET Deleted = ?

	where  SignalSource_ID = ?"$

	Try
		rtdb.ExecNonQuery2(qryStr, pars)
	Catch
		Log(LastException)
		Return True
	End Try
	Return False
End Sub


'============================================================================================
Public Sub deleteBandRecsForThisSigSrc(tfSigSrcID As Int, delete As Int) As Boolean
	Dim pars As List
	pars.Initialize
	
	pars.Add(delete)
	pars.Add(tfSigSrcID)

	Dim qryStr As String
		
	qryStr = _
	$"UPDATE tblMonitoredChannel 
	SET Deleted = ?
	Where SignalSource_ID = ?"$
	Try
		rtdb.ExecNonQuery2(qryStr, pars)
	Catch
		Log(LastException)
		Return True
	End Try
	Return False
End Sub



'============================================================================================
Public Sub deleteExceptionRecsForThisSigSrc(tfSigSrcID As Int, delete As Int) As Boolean
	Dim pars As List
	pars.Initialize
	
	pars.Add(delete)
	pars.Add(tfSigSrcID)


	Dim qryStr As String
		
	qryStr = _
	$"UPDATE tblChannelExceptionData 
	SET Deleted = ?
	Where SignalSource_ID = ?"$
	Try
		rtdb.ExecNonQuery2(qryStr, pars)
	Catch
		Log(LastException)
		Return True
	End Try
	Return False
End Sub




'============================================================================================
Public Sub deleteBandRecord(MonitoredCh_ID As Int, delete As Int) As Boolean
	Dim pars As List
	pars.Initialize
	
	pars.Add(delete)
	pars.Add(MonitoredCh_ID)

	Dim qryStr As String
		
	qryStr = _
	$"UPDATE tblMonitoredChannel 
	SET Deleted = ?
	Where MonitoredCh_ID = ?"$
	Try
		rtdb.ExecNonQuery2(qryStr, pars)
	Catch
		Log(LastException)
		Return True
	End Try
	Return False
End Sub


'============================================================================================
Public Sub getMonitoredChannelRecords(SignalSource_ID As Int) As ResultSet
	Dim dbRes As ResultSet

	Dim qryStr As String
	Dim argList As List
	argList.Initialize
	argList.Add(SignalSource_ID)
		
	qryStr = _
	$"SELECT
		MonitoredCh_ID,
		ChannelName,
		SignalSource_ID,
		DateCreatedUTC,
		ChannelStartFrequency_MHz,
		ChannelStopFrequency_MHz,
		ChannelMaxTH_1_dBm,
		ChannelMaxTH_2_dBm,
		ChannelMinTH_1_dBm,
		ChannelMinTH_2_dBm,
		ChannelMinTH_NoSignal_dBm,
		MinHysterisis_dBm,
		MaxHysterisis_dBm,
		Comment,
		NoPoints,
	    BandMarginMHz
	FROM tblMonitoredChannel
	WHERE SignalSource_ID = ? And Deleted = 0
	ORDER BY ChannelStartFrequency_MHz"$
	Try
		dbRes= rtdb.ExecQuery2(qryStr, argList)
	Catch
		Log(LastException)
	End Try
	Return dbRes
End Sub

'============================================================================================
Public Sub getMonitoredChannelRecord(MonitoredCh_ID As Int) As Object()
	Dim dbRes As ResultSet
	Dim items(16) As Object

	Dim qryStr As String
	Dim argList As List
	argList.Initialize
	argList.Add(MonitoredCh_ID)
		
	qryStr = _
	$"SELECT
		MonitoredCh_ID,
		ChannelName,
		SignalSource_ID,
		DateCreatedUTC,
		ChannelStartFrequency_MHz,
		ChannelStopFrequency_MHz,
		ChannelMaxTH_1_dBm,
		ChannelMaxTH_2_dBm,
		ChannelMinTH_1_dBm,
		ChannelMinTH_2_dBm,
		ChannelMinTH_NoSignal_dBm,
		MinHysterisis_dBm,
		MaxHysterisis_dBm,
		Comment,
		NoPoints,
	    BandMarginMHz
	FROM tblMonitoredChannel
	WHERE MonitoredCh_ID = ? And Deleted = 0
	ORDER BY ChannelStartFrequency_MHz"$
	Try
		dbRes = rtdb.ExecQuery2(qryStr, argList)
	Catch
		Log(LastException)
	End Try

	items(0) = dbRes.GetInt2(0)
	items(1) = dbRes.GetString2(1)
	items(2) = dbRes.GetInt2(2)
	items(3) = dbRes.GetString2(3)
	items(4) = dbRes.GetDouble2(4)
	items(5) = dbRes.GetDouble2(5)
	items(6) = dbRes.GetDouble2(6)
	items(7) = dbRes.GetDouble2(7)
	items(8) = dbRes.GetDouble2(8)
	items(9) = dbRes.GetDouble2(9)
	items(10) = dbRes.GetDouble2(10)
	items(11) = dbRes.GetDouble2(11)
	items(12) = dbRes.GetDouble2(12)
	items(13) = dbRes.GetString2(13)
	items(14) = dbRes.GetInt2(14)
	items(15) = dbRes.GetDouble2(15)

	Return items
End Sub


'============================================================================================
Public Sub howManyBandRecordsForThisSigSrc(SignalSource_ID As Int) As Int
	Dim dbRes As ResultSet

	Dim qryStr As String
	Dim argList As List
	argList.Initialize
	argList.Add(SignalSource_ID)
		
	qryStr = _
	$"Select count(MonitoredCh_ID)
	From tblMonitoredChannel
	Where SignalSource_ID = ?"$
	Try
		dbRes= rtdb.ExecQuery2(qryStr, argList)
	Catch
		Log(LastException)
	End Try
	Return dbRes.GetInt2(0)
End Sub


'============================================================================================
Public Sub insertSignalSourceRecord As Boolean
	Dim pars As List
	pars.Initialize
	
	pars.Add(Main.Band_Data.SigSrcCurrentRecord.SignalName)
	pars.Add(Main.Band_Data.SigSrcCurrentRecord.SignalBandStart_MHz)
	pars.Add(Main.Band_Data.SigSrcCurrentRecord.SignalBandStop_MHz)
	pars.Add(Main.Band_Data.SigSrcCurrentRecord.Comment)
	
	pars.Add(1)

	pars.Add(Main.Band_Data.SigSrcCurrentRecord.NoPoints)
	pars.Add(Main.Band_Data.SigSrcCurrentRecord.SampleTime)
	pars.Add(Main.Band_Data.SigSrcCurrentRecord.LNA_On)
	pars.Add("auto")
	pars.Add(Main.Band_Data.SigSrcCurrentRecord.Attenuator)
	pars.Add(Main.Band_Data.SigSrcCurrentRecord.RBW_Khz)
	pars.Add(Main.Band_Data.SigSrcCurrentRecord.RefLevel)
	pars.Add(Main.Band_Data.SigSrcCurrentRecord.NoSpectrumAvgs)
	pars.Add(Main.Band_Data.SigSrcCurrentRecord.RoutineDataFrequencyN)

	Dim qryStr As String
		
	qryStr = _
	$"INSERT INTO tblSignalSource( 
		SignalName, 
		SignalBandStart_MHz,
       	SignalBandStop_MHz,
       	Comment, 
       	InUse, 
       	NoPoints,
       	SampleTime,
       	LNA_On,
		LNA_Gain,
       	Attenuator,
       	RBW_Khz,
       	RefLevel,
       	NoSpectrumAvgs,
       	RoutineDataFrequencyN)
		VALUES(?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)"$
	Try
		rtdb.ExecNonQuery2(qryStr, pars)
	Catch
		Log(LastException)
		Return True
	End Try
	Return False
End Sub


'============================================================================================
Public Sub insertBandRecord As Boolean
	Dim pars As List
	pars.Initialize
	
	pars.Add(Main.tfChName.Text.Trim)
	pars.Add(Main.tfSigSrcID.Text.Trim)
	pars.Add(Main.tfStartMHz.Text.Trim)
	pars.Add(Main.tfStopMHz.Text.Trim)
	pars.Add(Main.tfMax1dBm.Text.Trim)
	pars.Add(Main.tfMax2dBm.Text.Trim)
	pars.Add(Main.tfMin1dBm.Text.Trim)
	pars.Add(Main.tfMin2dBm.Text.Trim)
	pars.Add(Main.tfNoSignaldBm.Text.Trim)
	pars.Add(Main.tfMinHystdBm.Text.Trim)
	pars.Add(Main.tfMaxHystdbM.Text.Trim)
	pars.Add(Main.tfComment.Text.Trim)
	pars.Add(Main.tfNoPoints.Text.Trim)
	pars.Add(Main.tfBandMarginMHz.Text.Trim)

	Dim qryStr As String
		
	qryStr = _
	$"INSERT INTO tblMonitoredChannel( 
		ChannelName,
		SignalSource_ID,
		ChannelStartFrequency_MHz,
		ChannelStopFrequency_MHz,
		ChannelMaxTH_1_dBm,
		ChannelMaxTH_2_dBm,
		ChannelMinTH_1_dBm,
		ChannelMinTH_2_dBm,
		ChannelMinTH_NoSignal_dBm,
		MinHysterisis_dBm,
		MaxHysterisis_dBm,
		Comment,
		NoPoints,
	    BandMarginMHz)
		VALUES(?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)"$
	Try
		rtdb.ExecNonQuery2(qryStr, pars)
	Catch
		Log(LastException)
		Return True
	End Try
	Return False
End Sub



'============================================================================================
Public Sub updateBandRecord(monCh As Int) As Boolean
	Dim pars As List
	pars.Initialize
	
	pars.Add(Main.tfChName.Text.Trim)
	pars.Add(Main.tfSigSrcID.Text.Trim)
	pars.Add(Main.tfStartMHz.Text.Trim)
	pars.Add(Main.tfStopMHz.Text.Trim)
	pars.Add(Main.tfMax1dBm.Text.Trim)
	pars.Add(Main.tfMax2dBm.Text.Trim)
	pars.Add(Main.tfMin1dBm.Text.Trim)
	pars.Add(Main.tfMin2dBm.Text.Trim)
	pars.Add(Main.tfNoSignaldBm.Text.Trim)
	pars.Add(Main.tfMinHystdBm.Text.Trim)
	pars.Add(Main.tfMaxHystdbM.Text.Trim)
	pars.Add(Main.tfComment.Text.Trim)
	pars.Add(Main.tfNoPoints.Text.Trim)
	pars.Add(Main.tfBandMarginMHz.Text.Trim)
	pars.Add(monCh)

	Dim qryStr As String
		
	qryStr = _
	$"UPDATE tblMonitoredChannel 
	SET	ChannelName = ?,
		SignalSource_ID = ?,
		ChannelStartFrequency_MHz = ?,
		ChannelStopFrequency_MHz = ?,
		ChannelMaxTH_1_dBm = ?,
		ChannelMaxTH_2_dBm = ?,
		ChannelMinTH_1_dBm = ?,
		ChannelMinTH_2_dBm = ?,
		ChannelMinTH_NoSignal_dBm = ?,
		MinHysterisis_dBm = ?,
		MaxHysterisis_dBm = ?,
		Comment = ?,
		NoPoints = ?,
		BandMarginMHz = ?
	Where MonitoredCh_ID = ?"$
	Try
		rtdb.ExecNonQuery2(qryStr, pars)
	Catch
		Log(LastException)
		Return True
	End Try
	Return False
End Sub


'============================================================================================
Public Sub insertBandExceptionRecord(bandIndx As Int) As Boolean
	Dim qryStr As String
	Dim item(13) As String

	'Convert sql parameters to strings
	item(0) = SaCom.bandLogRecord(bandIndx).UnixEpoch
	item(1) = SaCom.bandLogRecord(bandIndx).SignalSourceID
	item(2) = SaCom.bandLogRecord(bandIndx).MonitorChRec_ID
	item(3) = SaCom.bandLogRecord(bandIndx).TriggerType
	item(4) = SaCom.bandLogRecord(bandIndx).MeanSignal_dBm
	item(5) = SaCom.bandLogRecord(bandIndx).MaxSignal_dBm
	item(6) = SaCom.bandLogRecord(bandIndx).MinSignal_dBm
	item(7) = SaCom.bandLogRecord(bandIndx).ExcptnDurationMax1
	item(8) = SaCom.bandLogRecord(bandIndx).ExcptnDurationMax2
	item(9) = SaCom.bandLogRecord(bandIndx).ExcptnDurationMin1
	item(10) = SaCom.bandLogRecord(bandIndx).ExcptnDurationMin2
	item(11) = SaCom.bandLogRecord(bandIndx).ExcptnDurationNoSig
	item(12) = SaCom.bandExceptionIndex.tblBandExceptionDataRunIndex
	
	qryStr = _
	$"INSERT INTO tblChannelExceptionData(
	    UnixEpoch,
	    SignalSource_ID,
	    MonitorChRec_ID,
	    TriggerType,
	    MeanSignal_dBm,
	    MaxSignal_dBm,
	    MinSignal_dBm,
	    ExcptnDurationMax1,
	    ExcptnDurationMax2,
	    ExcptnDurationMin1,
	    ExcptnDurationMin2,
	    ExcptnDurationNoSig,
		tblBandExceptionDataRunIndex
		)
	VALUES(?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?) "$

	rtdb.BeginTransaction
	Try
		rtdb.ExecNonQuery2(qryStr, item)
	Catch
		#IF LOGGING_ON
		Log("clkDatabase Line 87" & LastException)
		#END IF		
		Return True
	End Try
	rtdb.TransactionSuccessful
	
	Return False
End Sub
	
	
'============================================================================================
'Create a record that is linked to the Signal source that records the
'start time for the run
Public Sub insertRunExceptionIndex As Boolean
	Dim qryStr As String
	Dim item(6) As String

	'Convert sql parameters to strings
	item(0) = SaCom.bandExceptionIndex.StartUnixEpoch
	item(1) = SaCom.bandExceptionIndex.StartDateTimeUTC
	item(2) = SaCom.bandExceptionIndex.SignalSource_ID
	item(3) = SaCom.bandExceptionIndex.Comment
	item(4) = SaCom.bandExceptionIndex.Details
	item(5) = SaCom.bandExceptionIndex.Deleted
	
	qryStr = _
	$"INSERT INTO tblBandExceptionRunIndex
	(
		StartUnixEpoch,
		StartDateTimeUTC,
		SignalSource_ID,
		Comment,
		Details,
		Deleted
	)
	VALUES(?, ?, ?, ?, ?, ? )"$

	rtdb.BeginTransaction
	Try
		rtdb.ExecNonQuery2(qryStr, item)
	Catch
		#IF LOGGING_ON
		Log("clkDatabase Line 87" & LastException)
		#END IF		
		Return True
	End Try
	rtdb.TransactionSuccessful
	
	Dim rset As ResultSet
	
	qryStr = _
	$"SELECT
		tblBandExceptionDataRunIndex,
		StartUnixEpoch,
		StartDateTimeUTC,
		SignalSource_ID,
		Comment,
		Details,
		Deleted
	From tblBandExceptionRunIndex
	Order by tblBandExceptionDataRunIndex DESC
	limit 1"$
	
	Try
		rset = rtdb.ExecQuery(qryStr)
	Catch
		Log("DB error getting tblBandExceptionRunIndex")
		Return True
	End Try
	
	Do While rset.NextRow
		SaCom.bandExceptionIndex.tblBandExceptionDataRunIndex = rset.GetInt2(0)
		SaCom.bandExceptionIndex.StartUnixEpoch = rset.GetLong2(1)
		SaCom.bandExceptionIndex.StartDateTimeUTC = rset.GetString2(2)
		SaCom.bandExceptionIndex.SignalSource_ID = rset.GetInt2(3)
		SaCom.bandExceptionIndex.Comment = rset.GetString2(4)
		SaCom.bandExceptionIndex.Details = rset.GetString2(5)
		SaCom.bandExceptionIndex.Deleted = rset.GetInt2(6)
	Loop
	Return False
End Sub
	
  
  	
'============================================================================================
'Create a record that is linked to the Signal source that records the
'start time for the run
Public Sub getRunExceptionIndexsForSigSource As List
	Dim qryStr As String

	Dim tableList As List
	tableList.Initialize

	'Convert sql parameters to strings
	Dim parList As List 
	parList.Initialize	
	
	parList.Add(Main.Band_Data.SigSrcCurrentRecord.SignalSource_ID)

	Dim rset As ResultSet
	
	qryStr = _
	$"SELECT
		tblBandExceptionDataRunIndex,
		StartDateTimeUTC,
		Comment,
		Details
	From tblBandExceptionRunIndex
	Where SignalSource_ID = ?
	Order by tblBandExceptionDataRunIndex DESC"$
	
	Try
		rset = rtdb.ExecQuery2(qryStr, parList)
	Catch
		Log("DB error getting tblBandExceptionRunIndex")
		Return tableList
	End Try
	
	Do While rset.NextRow
		Dim itemOutList(4) As Object
		itemOutList(0) = rset.GetInt2(0)
		itemOutList(1) = rset.GetString2(1)
		itemOutList(2) = rset.GetString2(2)
		itemOutList(3) = rset.GetString2(3)
		tableList.Add(itemOutList)
	Loop
	Return tableList
End Sub
	 
	 
	
'============================================================================================
'Create a record that is linked to the Signal source that records the
'start time for the run
Public Sub updateRunExceptionIndexEndTime As Boolean
	Dim qryStr As String
	Dim item(3) As String

	'Convert sql parameters to strings
	item(0) = SaCom.bandExceptionIndex.EndUnixEpoch
	item(1) = SaCom.bandExceptionIndex.EndDateTimeUTC
	item(2) = SaCom.bandExceptionIndex.tblBandExceptionDataRunIndex
	
	'Convert sql parameters to strings
	Dim parList As List
	parList.Initialize
	parList.Add(item)
	
	qryStr = _
	$"UPDATE tblBandExceptionRunIndex
	  SET EndUnixEpoch = ?,
		  EndDateTimeUTC = ?
      WHERE SignalSource_ID = ?"$

	rtdb.BeginTransaction
	Try
		rtdb.ExecNonQuery2(qryStr, parList)
	Catch
		#IF LOGGING_ON
		Log("clkDatabase Line 87" & LastException)
		#END IF		
		Return True
	End Try
	rtdb.TransactionSuccessful
	
	Dim rset As ResultSet
	
	qryStr = _
	$"SELECT
		tblBandExceptionDataRunIndex,
		StartUnixEpoch,
		StartDateTimeUTC,
		SignalSource_ID,
		Comment,
		Details,
		Deleted
	From tblBandExceptionRunIndex
	Order by tblBandExceptionDataRunIndex DESC
	limit 1"$
	
	Try
		rset = rtdb.ExecQuery(qryStr)
	Catch
		Log("DB error getting tblBandExceptionRunIndex")
		Return True
	End Try
	
	Do While rset.NextRow
		SaCom.bandExceptionIndex.tblBandExceptionDataRunIndex = rset.GetInt2(0)
		SaCom.bandExceptionIndex.StartUnixEpoch = rset.GetLong2(1)
		SaCom.bandExceptionIndex.StartDateTimeUTC = rset.GetString2(2)
		SaCom.bandExceptionIndex.SignalSource_ID = rset.GetInt2(3)
		SaCom.bandExceptionIndex.Comment = rset.GetString2(4)
		SaCom.bandExceptionIndex.Details = rset.GetString2(5)
		SaCom.bandExceptionIndex.Deleted = rset.GetInt2(6)
	Loop
	Return False
End Sub
	
  	 